# Reference
<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

**Resource types**

* [`slurm_account`](#slurm_account): Puppet type that manages a SLURM account
* [`slurm_cluster`](#slurm_cluster): Puppet type that manages a SLURM cluster
* [`slurm_config`](#slurm_config): Abstract type to configure other SLURM types
* [`slurm_license`](#slurm_license): Puppet type that manages a SLURM software resource
* [`slurm_qos`](#slurm_qos): Puppet type that manages a SLURM QOS
* [`slurm_reservation`](#slurm_reservation): Puppet type that manages a SLURM Reservation
* [`slurm_user`](#slurm_user): Puppet type that manages a SLURM user
* [`slurmdbd_conn_validator`](#slurmdbd_conn_validator): Verify that a connection can be successfully established between a node and the slurmdbd server.  Its primary use is as a precondition to pre

## Resource types

### slurm_account

Puppet type that manages a SLURM account

#### Examples

##### Add SLURM account

```puppet
slurm_account { 'staff on cluster':
  ensure    => 'present',
  max_jobs  => 1000,
  priority  => 9999,
}

@example Add SLURM account
  slurm_account { 'staff:cluster':
    ensure    => 'present',
    max_jobs  => 1000,
    priority  => 9999,
  }
```

#### Properties

The following properties are available in the `slurm_account` type.

##### `ensure`

Valid values: present, absent

The basic property that the resource should be in.

Default value: present

##### `organization`

Organization

##### `parent_name`

Account parent name

##### `description`

Description

##### `default_qos`

DefaultQOS

##### `fairshare`

Fairshare number or "parent"

Default value: 1

##### `grp_tres_mins`

GrpTRESMins

Default value: absent

##### `grp_tres_run_mins`

GrpTRESRunMins

Default value: absent

##### `grp_tres`

GrpTRES

Default value: absent

##### `grp_jobs`

GrpJobs

Default value: absent

##### `grp_jobs_accrue`

GrpJobsAccrue

Default value: absent

##### `grp_submit_jobs`

GrpSubmitJobs

Default value: absent

##### `grp_wall`

GrpWall

Default value: absent

##### `max_tres_mins_per_job`

MaxTresMinsPerJob

Default value: absent

##### `max_tres_per_job`

MaxTresPerJob

Default value: absent

##### `max_tres_per_node`

MaxTresPerJob

Default value: absent

##### `max_jobs`

MaxJobs

Default value: absent

##### `max_jobs_accrue`

MaxJobsAccrue

Default value: absent

##### `max_submit_jobs`

MaxSubmitJobs

Default value: absent

##### `max_wall_duration_per_job`

MaxWallDurationPerJob

Default value: absent

##### `priority`

Priority

##### `qos`

QOS, undefined will inherit parent QOS

#### Parameters

The following parameters are available in the `slurm_account` type.

##### `name`

namevar

Account name

##### `account`

Account name

##### `cluster`

Cluster name

### slurm_cluster

Puppet type that manages a SLURM cluster

#### Examples

##### Add a SLURM cluster

```puppet
slurm_cluster { 'test':
  ensure => 'present',
}
```

#### Properties

The following properties are available in the `slurm_cluster` type.

##### `ensure`

Valid values: present, absent

The basic property that the resource should be in.

Default value: present

##### `features`

Features

Default value: absent

##### `federation`

Federation

Default value: absent

##### `fed_state`

Valid values: active, inactive, drain, drain_remove

FedState

#### Parameters

The following parameters are available in the `slurm_cluster` type.

##### `name`

namevar

cluster name

##### `flags`

Flags

### slurm_config

Abstract type to configure other SLURM types

#### Parameters

The following parameters are available in the `slurm_config` type.

##### `name`

namevar

The name of the resource

##### `install_prefix`

The path to SLURM install prefix

### slurm_license

Puppet type that manages a SLURM software resource

#### Examples

##### Add SLURM software resource

```puppet
slurm_license { 'matlab@host':
  ensure  => 'present',
  count   => 100,
}
slurm_license { 'matlab@host for linux':
  ensure          => 'present',
  percent_allowed => 100,
}
```

#### Properties

The following properties are available in the `slurm_license` type.

##### `ensure`

Valid values: present, absent

The basic property that the resource should be in.

Default value: present

##### `description`

Description

##### `count`

Count

##### `server_type`

ServerType

Default value: absent

##### `percent_allowed`

PercentAllowed

#### Parameters

The following parameters are available in the `slurm_license` type.

##### `name`

namevar

Resource name

##### `resource_name`

Resource name

##### `server`

Server

##### `cluster`

Cluster

##### `type`

Resource type, read-only

Default value: License

### slurm_qos

Puppet type that manages a SLURM QOS

#### Examples

##### Add SLURM QOS

```puppet
slurm_qos { 'high':
  ensure            => 'present',
  flags             => ['DenyOnLimit','RequiresReservation'],
  grace_time        => 300,
  grp_tres          => { 'node' => 40 },
  max_tres_per_user => { 'node' => 20 },
  max_wall          => '2-00:00:00',
  priority          => 2000000,
}
```

#### Properties

The following properties are available in the `slurm_qos` type.

##### `ensure`

Valid values: present, absent

The basic property that the resource should be in.

Default value: present

##### `description`

Description

##### `flags`

Valid values: DenyOnLimit, EnforceUsageThreshold, NoReserve, PartitionMaxNodes, PartitionMinNodes, OverPartQOS, PartitionTimeLimit, RequiresReservation, NoDecay, UsageFactorSafe, absent

Flags

Default value: absent

##### `grace_time`

GraceTime

Default value: 0

##### `grp_tres_mins`

GrpTRESMins

Default value: absent

##### `grp_tres_run_mins`

GrpTRESRunMins

Default value: absent

##### `grp_tres`

GrpTRES

Default value: absent

##### `grp_jobs`

GrpJobs

Default value: absent

##### `grp_jobs_accrue`

GrpJobsAccrue

Default value: absent

##### `grp_submit_jobs`

GrpSubmitJobs

Default value: absent

##### `grp_wall`

GrpWall

Default value: absent

##### `max_tres_mins`

MaxTresMins

Default value: absent

##### `max_tres_per_account`

MaxTresPerAccount

Default value: absent

##### `max_tres_per_job`

MaxTresPerJob

Default value: absent

##### `max_tres_per_node`

MaxTresPerNode

Default value: absent

##### `max_tres_per_user`

MaxTresPerUser

Default value: absent

##### `max_jobs_per_account`

MaxJobsPerAccount

Default value: absent

##### `max_jobs_per_user`

MaxJobsPerUser

Default value: absent

##### `max_submit_jobs_per_account`

MaxSubmitJobsPerAccount

Default value: absent

##### `max_submit_jobs_per_user`

MaxSubmitJobsPerUser

Default value: absent

##### `max_wall`

MaxWall

Default value: absent

##### `min_prio_threshold`

MinPrioThreshold

Default value: absent

##### `min_tres_per_job`

MinTRESPerJob

Default value: absent

##### `preempt`

Preempt

Default value: absent

##### `preempt_mode`

Valid values: cluster, cancel, checkpoint, requeue

PreemptMode

Default value: cluster

##### `preempt_exempt_time`

PreemptExemptTime

Default value: absent

##### `priority`

Priority

Default value: 0

##### `usage_factor`

UsageFactor

Default value: 1.000000

##### `usage_threshold`

UsageThreshold

Default value: absent

#### Parameters

The following parameters are available in the `slurm_qos` type.

##### `name`

namevar

QOS name

### slurm_reservation

Puppet type that manages a SLURM Reservation

#### Examples

##### Add SLURM Reservation

```puppet
slurm_reservation { 'maint':
  ensure     => 'present',
  start_time => 'now',
  duration   => '02:00:00',
  users      => ['root'],
  flags      => ['maint','ignore_jobs'],
  nodes      => 'ALL',
}
```

#### Properties

The following properties are available in the `slurm_reservation` type.

##### `ensure`

Valid values: present, absent

The basic property that the resource should be in.

Default value: present

##### `accounts`

Accounts

##### `burst_buffer`

BurstBuffer

##### `core_cnt`

CoreCnt

##### `licenses`

Licenses

##### `node_cnt`

NodeCnt

##### `nodes`

Nodes

##### `start_time`

StartTime

##### `end_time`

EndTime

##### `duration`

Duration

##### `partition_name`

PartitionName

##### `flags`

Flags

##### `features`

Features

##### `users`

Users

##### `tres`

TRES

#### Parameters

The following parameters are available in the `slurm_reservation` type.

##### `name`

namevar

Reservation name

### slurm_user

Puppet type that manages a SLURM user

#### Examples

##### Add SLURM user under account 'bar' on cluster 'test'

```puppet
slurm_user { 'foo under bar on test':
  ensure    => 'present',
  max_jobs  => 1000,
  priority  => 9999,
}
```

##### Add SLURM user under account 'bar' on cluster 'test'

```puppet
slurm_user { 'foo:bar:test':
  ensure    => 'present',
  max_jobs  => 1000,
  priority  => 9999,
}
```

#### Properties

The following properties are available in the `slurm_user` type.

##### `ensure`

Valid values: present, absent

The basic property that the resource should be in.

Default value: present

##### `admin_level`

Valid values: None, Operator, Administrator

AdminLevel

Default value: None

##### `default_account`

DefaultAccount

##### `default_qos`

DefaultQOS

##### `fairshare`

Fairshare number or "parent"

Default value: 1

##### `grp_tres_mins`

GrpTRESMins

Default value: absent

##### `grp_tres_run_mins`

GrpTRESRunMins

Default value: absent

##### `grp_tres`

GrpTRES

Default value: absent

##### `grp_jobs`

GrpJobs

Default value: absent

##### `grp_jobs_accrue`

GrpJobsAccrue

Default value: absent

##### `grp_submit_jobs`

GrpSubmitJobs

Default value: absent

##### `grp_wall`

GrpWall

Default value: absent

##### `max_tres_mins_per_job`

MaxTresMinsPerJob

Default value: absent

##### `max_tres_per_job`

MaxTresPerJob

Default value: absent

##### `max_tres_per_node`

MaxTresPerJob

Default value: absent

##### `max_jobs`

MaxJobs

Default value: absent

##### `max_jobs_accrue`

MaxJobsAccrue

Default value: absent

##### `max_submit_jobs`

MaxSubmitJobs

Default value: absent

##### `max_wall_duration_per_job`

MaxWallDurationPerJob

Default value: absent

##### `priority`

Priority

##### `qos`

QOS, undefined will inherit parent QOS

#### Parameters

The following parameters are available in the `slurm_user` type.

##### `name`

namevar

User name

##### `user`

User name

##### `account`

Account name

##### `cluster`

Cluster name

### slurmdbd_conn_validator

Verify that a connection can be successfully established between a node
and the slurmdbd server.  Its primary use is as a precondition to
prevent configuration changes from being applied if the slurmdbd
server cannot be reached.

#### Properties

The following properties are available in the `slurmdbd_conn_validator` type.

##### `ensure`

Valid values: present, absent

The basic property that the resource should be in.

Default value: present

#### Parameters

The following parameters are available in the `slurmdbd_conn_validator` type.

##### `name`

namevar

An arbitrary name used as the identity of the resource.

##### `timeout`

The max number of seconds that the validator should wait before giving up and deciding that slurmdbd is not running; defaults to 30 seconds.

Default value: 30

